/* Corsair - Qtopia RSS/Podcast Aggregator
 *
 * Copyright (C) 2005 Joe Rumsey
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef COSAIR_H__
#define CORSAIR_H__

#include "feeds.h"
#include "log.h"
#include "version.h"

#include <qmainwindow.h>
#include <qlistview.h>

class QTextBrowser;
class Download;
class Feed;
class FeedShow;
class QPEApplication;
class QProgressBar;
class FeedInfo;

class CorsairWindow : public QMainWindow
{
	Q_OBJECT;

private:

	QToolBar *m_bar;
	QListView *m_list;
	
	FeedList m_feeds;
	
	FeedShow *m_showDL;

	QList<Download> m_runningDownloads;
	QList<Download> m_pendingDownloads;

	QWidget *m_infoDialog;
	QTextBrowser *m_infoBrowser;
	QPushButton *m_infoNextButton;
	QListViewItem *m_infoItem;
	QPushButton *m_infoActionButton;
	QPopupMenu *m_menuPopup;
	QProgressBar *m_progress;
	FeedInfo     *m_feedInfoDialog;
	QPEApplication *m_app;

public:
	CorsairWindow(QPEApplication *app, QWidget *parent = NULL, const char *name = NULL);
	~CorsairWindow();

	void showEvent(QShowEvent *ev);
	void initCorsair();

	void ShowFeeds();
	
	QString option(QString name, QString defval);
	void setOption(QString name, QString value);

    void grabItem(QListViewItem *item);

	QString homeDir();
	QString topDir(bool create = true);
	QString feedDir();
	QString showsDir();
	QString version() { return QString(CORSAIRVERSION); }

	Download *startFeedDownload(Feed *f, bool isNew);

	void view(QListViewItem *item);
	void play(FeedShow *show);
	void viewFeedInfo(QListViewItem *feedItem);
	void saveFeeds();
	bool startEnclosureDownload(const QString &url);
	void startShowDownload(FeedShow *s);

public slots:
    void cleanup();
	void newFeed();
	void downloadComplete(Download *dl, int result);
	void doubleClick(QListViewItem *item);
	void returnPressed(QListViewItem *item);
	void grabSelected();
	void setStatus(const QString &);
	void viewSelected();
	void delSelected();
	void progressUpdate(Download *dl, int read);
	void getAllFeeds();
	void infoPrev();
	void infoDownload();
	void infoNext();
	void infoClosed();
	void infoBrowse();
	void about();
	void help();
	void playSelected();
	void menu();
	void loadNewFile();
	void viewDownloads();
	void preferences();
	void expand(QListViewItem *item);
	void feedInfoDestroyed();
	void checkOldDirectories(QString newdir);

signals:
	void statusUpdate(const QString &);
};

#endif
