/* Corsair - Qtopia RSS/Podcast Aggregator
 *
 * Copyright (C) 2005 Joe Rumsey
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef DOWNLOAD_H__
#define DOWNLOAD_H__

#include <qsocket.h>
#include <qurl.h>

class CorsairWindow;

class Download : public QSocket {
	Q_OBJECT;

private:
	QUrl m_url;
	CorsairWindow *m_parent;
	QString m_fileName;
	QFile m_file;
	bool m_headersComplete;
	bool m_chunked;
	int m_chunkRemaining;
	bool m_binary;
	int m_expectedLen;
	int m_totalRead;
	bool m_gotResponse;
	bool m_redirect;
	QString m_redirectLocation;
	bool m_complete;

	void *m_param;

	void init();

public:
	enum Result {
		Success,
		NoData,
		CantConnect,
		NotFound,
		Error
	};

	Download(CorsairWindow *parent, QUrl &url, QString &fileName, void *param);
	~Download();

	void go();

	QString fileName() { return m_fileName; }

	void *param() { return m_param; }

	int expectedLen() { return m_expectedLen; }
	void complete(Result res);
	void openFile();

public slots:
	void readyRead();
	void error(int e);
	void connected();
	void hostFound();
	void connectionClosed();
  
signals:
	void downloadComplete(Download *, int);
	void statusUpdate(const QString &);
	void progressUpdate(Download *, int);
};

#endif
