/* Corsair - Qtopia RSS/Podcast Aggregator
 *
 * Copyright (C) 2005 Joe Rumsey
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef FEEDPARSER_H__
#define FEEDPARSER_H__

#define MAX_NEST 30

class FeedParser : public QXmlDefaultHandler
{
private:
	Feed *m_feed;
	int m_level;
	QString m_elementName[MAX_NEST];
	QString m_elementData[MAX_NEST];
	QString m_elemTitle[MAX_NEST];
	QString m_elemLink[MAX_NEST];

	QString m_enclosure;
	QString m_description;

	bool m_reportedError;
public:
	FeedParser(Feed *feed) {
		m_feed = feed;
		m_level = -1;
		m_reportedError = false;
	}

	bool startDocument() {
		LOG(3, "startDocument\n");
		return true;
	}
	bool startElement(const QString &namespaceURI,
					  const QString &localName,
					  const QString &name,
					  const QXmlAttributes &attr)
	{    
		LOG(3, "startElement: %s\n", (const char *)name);
		int i;
		for(i = 0; i < attr.length(); i++) {
			LOG(3, "  %s=%s\n", (const char *)attr.qName(i), (const char *)attr.value(i));
		}
		
		++m_level;
		m_elementName[m_level] = name;
		m_elementData[m_level] = "";
		
		if(name == "item") {
			m_elemTitle[m_level] = "";
			m_enclosure = "";
			m_description = "";
		} else if(name == "enclosure") {
			m_enclosure = attr.value("url");
		}

		return true;
	}

	bool characters(const QString &data) {
		m_elementData[m_level] += data;
		return true;
	}

	bool endElement(const QString &namespaceURI,
					const QString &localName,
					const QString &name) 
	{
		LOG(3, "endElement %s\n", (const char *)name);
		if(m_elementName[m_level] == "item") {
			m_feed->addShow(m_elemTitle[m_level], 
							m_enclosure, 
							m_description, 
							m_elemLink[m_level]);
		} else if(m_elementName[m_level] == "channel") {
			m_feed->setName(m_elemTitle[m_level]);
		} else if(m_elementName[m_level] == "title") {
			m_elemTitle[m_level - 1] = m_elementData[m_level];
		} else if(m_elementName[m_level] == "description") {
			m_description = m_elementData[m_level];
		} else if(m_elementName[m_level] == "link") {
			m_elemLink[m_level -1] = m_elementData[m_level];
		}

		--m_level;
		return true;
	}

	bool warning(const QXmlParseException &e) {
		LOG(1, "warning\n");
		return true;
	}

	bool error(const QXmlParseException &e) {
		LOG(1, "error\n");
		return true;
	}

	bool fatalError(const QXmlParseException &e) {
		LOG(1, "fatal error: %s\n", (const char *)e.message());

		return false;
	}

};

#endif
