/* Corsair - Qtopia RSS/Podcast Aggregator
 *
 * Copyright (C) 2005 Joe Rumsey
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef FEEDS_H__
#define FEEDS_H__

#include <qstring.h>
#include <qlist.h>
#include <qxml.h>
#include <qlistview.h>
#include <qdatetime.h>
#include "log.h"
#include "download.h"

enum LIST_COLUMNS {
	NAME_COLUMN,
	DATE_COLUMN
};

enum ITEM_TYPE {
	IT_GROUP,
	IT_FEED,
	IT_SHOW
};

class CorsairWindow;

class FeedList;
class GroupListViewItem;
class FeedListViewItem;
class ShowListViewItem;
class Feed;

class FeedShow
{
public:
	FeedShow(Feed *feed,
			 const QString &title,
			 const QString &enclosure,
			 const QString &description,
			 const QString &link,
			 bool isNew,
			 int index);
		
	~FeedShow() {
		// FIXME - if there's a download in progress, the download needs to
		// be signalled.  Or something.
	}

	Feed *m_feed;

	QString m_title;
	QString m_enclosure;
	QString m_description;
	QString m_link;

	bool m_isNew;
	int m_index;
	
	bool m_haveLocalCopy;

	ShowListViewItem *m_listItem;

	QString fileName() { 
		int loc = m_enclosure.findRev('/') + 1;
		QString f = m_enclosure.right(m_enclosure.length() - loc);
		return f;
	}

	QString fullLocalPath(QString &subDir);
	void setHaveLocalCopy(bool haveit);

	int getIndex() { return m_index; }
	void setIndex(int val) { m_index = val; }

};

class Feed : public QObject
{
	Q_OBJECT;

private:
	FeedListViewItem *m_feedItem;

public:
	QString m_name;
	QString m_url;
	QDateTime m_lastGrab;

	QList<FeedShow> m_shows;

	QXmlAttributes m_attr;
	FeedList *m_list;
	int m_showIndex;

	bool m_loaded;
	bool m_brokenXML;

	Feed(FeedList *list, const QString &name, const QString &url);
	Feed(FeedList *list, const QXmlAttributes &attr);
	~Feed();

	QString Link();

	QPixmap *chooseIcon(FeedShow *show);

	void updateListItem(QListViewItem *g = NULL);
	void addShow(const QString &title, const QString &enclosure, const QString &description,
				 const QString &link);

	QString fileName() {
		return dirName() + ".xml";
	}

	QString dirName();

	void Load(QString filename);
	void clear();

	void setName(const QString &name);
	void rename(const QString &name);

 public slots:
    void downloadComplete(Download *dl, int result);

};

class FeedList
{
private:
	QString m_filename;
	QList<Feed> m_feeds;
	CorsairWindow *m_owner;
	GroupListViewItem *m_groupItem;
	bool m_showsAreNew;


public:
	FeedList(CorsairWindow *owner);
	~FeedList();
  
	void Clear();
	void Load(const QString &file);

	Feed *addFeed(const QString &name, const QString &url);
	void AddFeed(const QXmlAttributes &attr);
	void addToList(QListView *l);
	void save(QString file = QString(""));
	void reload(Feed *f, const QString &filename, bool showsAreNew);

	Feed *GetFeed(QListViewItem *item);
	FeedShow *GetShow(QListViewItem *item);

	ITEM_TYPE getType(QListViewItem *item);

	CorsairWindow *owner() { return m_owner; }

	bool showsAreNew() { return m_showsAreNew; }

	void delFeed(QListViewItem *item);

	QList<Feed> *feedList() { return &m_feeds; }

	GroupListViewItem *groupItem() { return m_groupItem; }
	FeedShow *findEnclosureShow(const QString &url);
};

#endif
