
#include "corsair.h"
#include "info.h"
#include "log.h"

#include <qlayout.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qhbox.h>
#include <qcheckbox.h>

#include <qtopia/qprocess.h>
#include <qtopia/qcopenvelope_qws.h>

InfoBrowser::InfoBrowser(CorsairWindow *main, QWidget *parent) :
	QTextBrowser(parent),
	m_main(main)
{
}

void InfoBrowser::setSource(const QString &str)
{
	if(m_main->startEnclosureDownload(str)) {
		// Use internal download engine for anything marked as an enclosure
		return;
	}

	LOG(1, "setSource(%s)\n", (const char *)str);
	QCString browser = (const char *)QString("QPE/Application/" + m_main->option("browser", "opera"));
	QCString command = "setDocument(QString)";

	QCopEnvelope e(browser, command);
	e << str;

	QCopEnvelope r(browser, "raise()");
}
	
FeedInfo::FeedInfo(CorsairWindow *main) :
	QDialog(main, "FeedInfo", false, WStyle_Customize | WStyle_DialogBorder | WDestructiveClose)
{
	m_feed = NULL;
	m_main = main;

	QVBoxLayout *vbox = new QVBoxLayout(this);
	vbox->setAutoAdd(true);

	QHBox *h = new QHBox(this);

	QLabel *l = new QLabel(tr("Name:"), h);
	l->setMinimumWidth(40);
	m_nameEdit = new QLineEdit("", h, "nameEdit");

	h = new QHBox(this);
	l = new QLabel(tr("URL:"), h);
	l->setMinimumWidth(40);
	m_urlEdit = new QLineEdit("", h, "urlEdit");

	h = new QHBox(this);
	m_brokenXMLCB = new QCheckBox("Broken XML", h);

	m_text = new InfoBrowser(m_main, this);
}
	
FeedInfo::~FeedInfo()
{
	LOG(1, "FeedInfo::~FeedInfo()\n");
}

void FeedInfo::setFeed(Feed *f)
{
	m_feed = f;
	m_brokenXMLCB->setChecked(m_feed->m_brokenXML);
	m_nameEdit->setText(m_feed->m_name);
	m_urlEdit->setText(m_feed->m_url);
	setCaption(m_feed->m_name);

	QString text = "<html>\n";
	FeedShow *s;
	bool first = true;
	for(s = m_feed->m_shows.first(); s; s = m_feed->m_shows.next()) {
		if(!first) {
			text += "<hr>";
		} else {
			first = false;
		}
		text += "<H3><a href=\"" + s->m_link + "\">" + s->m_title + "</a></H3>\n";
		if(!s->m_enclosure.isEmpty()) {
			text += "<a href=\"" + s->m_enclosure + "\">[Enclosure]</a><br>\n";
		}
		text += s->m_description + "\n";
	}
	text += "</html>\n";

	m_text->setText(text);
}

void FeedInfo::accept()
{
	if(m_feed) {
		LOG(1, "accept, name=%s, url=%s\n", 
			(const char *)m_nameEdit->text(), 
			(const char *)m_urlEdit->text());

		m_feed->rename(m_nameEdit->text());
		m_feed->m_url = m_urlEdit->text();
		bool reload = m_feed->m_brokenXML != m_brokenXMLCB->isChecked();
		m_feed->m_brokenXML = m_brokenXMLCB->isChecked();

		if(reload) {
			m_feed->Load("");
		}
		m_feed->updateListItem();
		m_main->saveFeeds();
	}
	QDialog::accept();
}
